@@ -2,30 +2,28 @@
 #
 # SPDX-License-Identifier: MPL-2.0
 from pathlib import Path
-from typing import List, Tuple, Union
+from typing import List, Tuple
 
 import optuna
 import pandas as pd
 import structlog
-from openstef.data_classes.prediction_job import PredictionJobDataClass
 
 from openstef.data_classes.model_specifications import ModelSpecificationDataClass
+from openstef.data_classes.prediction_job import PredictionJobDataClass
 from openstef.exceptions import (
     InputDataInsufficientError,
     InputDataWrongColumnOrderError,
 )
 from openstef.feature_engineering.feature_applicator import TrainFeatureApplicator
+from openstef.metrics.reporter import Reporter
 from openstef.model.model_creator import ModelCreator
 from openstef.model.objective import RegressorObjective
 from openstef.model.objective_creator import ObjectiveCreator
+from openstef.model.serializer import MLflowSerializer
 from openstef.pipeline.train_model import (
     DEFAULT_TRAIN_HORIZONS,
     train_model_pipeline_core,
 )
-
-# This is required to disable the default optuna logger and pass the logs to our own
-# structlog logger
-from openstef.model.serializer import MLflowSerializer
 from openstef.validation import validation
 
 optuna.logging.enable_propagation()  # Propagate logs to the root logger.
@@ -41,7 +39,8 @@ TIMEOUT: int = 600  # Stop study after the given number of second(s).
 def optimize_hyperparameters_pipeline(
     pj: PredictionJobDataClass,
     input_data: pd.DataFrame,
-    trained_models_folder: Union[str, Path],
+    mlflow_tracking_uri: str,
+    artifact_folder: str,
     horizons: List[float] = DEFAULT_TRAIN_HORIZONS,
     n_trials: int = N_TRIALS,
 ) -> dict:
@@ -52,7 +51,8 @@ def optimize_hyperparameters_pipeline(
     Args:
         pj (PredictionJobDataClass): Prediction job
         input_data (pd.DataFrame): Raw training input data
-        trained_models_folder (Path): Path where trained models are stored
+        mlflow_tracking_uri (str): Path/Uri to mlflow service
+        artifact_folder (str): Path where artifacts, such as trained models, are stored
         horizons (List[float]): horizons for feature engineering.
         n_trials (int, optional): The number of trials. Defaults to N_TRIALS.
 
@@ -70,17 +70,27 @@ def optimize_hyperparameters_pipeline(
         )
 
     # Validate and clean data
-    validated_data = validation.clean(validation.validate(pj["id"], input_data))
+    validated_data = validation.drop_target_na(
+        validation.validate(pj["id"], input_data, pj["flatliner_treshold"])
+    )
 
     # Check if sufficient data is left after cleaning
-    if not validation.is_data_sufficient(validated_data):
+    if not validation.is_data_sufficient(
+        validated_data, pj["completeness_treshold"], pj["minimal_table_length"]
+    ):
         raise InputDataInsufficientError(
-            f"Input data is insufficient for {pj['name']} "
-            f"after validation and cleaning"
+            f"Input data is insufficient for {pj['name']} after validation and cleaning"
         )
 
+    if pj.default_modelspecs:
+        feature_names = (pj.default_modelspecs.feature_names,)
+        feature_modules = pj.default_modelspecs.feature_modules
+    else:
+        feature_names = None
+        feature_modules = []
+
     validated_data_with_features = TrainFeatureApplicator(
-        horizons=horizons
+        horizons=horizons, feature_names=feature_names, feature_modules=feature_modules
     ).add_features(validated_data, pj=pj)
 
     # Adds additional proloaf features to the input data, historic_load (equal to the load, first column)
@@ -96,7 +106,7 @@ def optimize_hyperparameters_pipeline(
         validated_data_with_features = validated_data_with_features[new_cols]
 
     # Create serializer
-    serializer = MLflowSerializer(trained_models_folder)
+    serializer = MLflowSerializer(mlflow_tracking_uri=mlflow_tracking_uri)
 
     # Create objective (NOTE: this is a callable class)
     objective = ObjectiveCreator.create_objective(model_type=pj["model"])
@@ -118,7 +128,7 @@ def optimize_hyperparameters_pipeline(
         best_hyperparams.update(quantiles=pj["quantiles"])
 
     # model specification
-    modelspecs = ModelSpecificationDataClass(
+    model_specs = ModelSpecificationDataClass(
         id=pj["id"],
         feature_names=list(validated_data_with_features.columns),
         hyper_params=best_hyperparams,
@@ -127,21 +137,24 @@ def optimize_hyperparameters_pipeline(
     # If the model type is quantile, train a model with the best parameters for all quantiles
     # (optimization is only done for quantile 0.5)
     if objective.model.can_predict_quantiles:
-        best_model, report, modelspecs = train_model_pipeline_core(
-            pj=pj, input_data=input_data, modelspecs=modelspecs
+        best_model, report, modelspecs, _ = train_model_pipeline_core(
+            pj=pj, input_data=input_data, model_specs=model_specs
         )
 
-    # Save model
+    # Save model and report. Report is always saved to MLFlow and optionally to disk
+    report = objective.create_report(model=best_model)
     serializer.save_model(
-        best_model,
-        pj=pj,
-        modelspecs=modelspecs,
-        report=objective.create_report(model=best_model),
+        model=best_model,
+        experiment_name=str(pj["id"]),
+        model_type=pj["model"],
+        model_specs=model_specs,
+        report=report,
         phase="Hyperparameter_opt",
         trials=objective.get_trial_track(),
         trial_number=study.best_trial.number,
     )
-
+    if artifact_folder:
+        Reporter.write_report_to_disk(report=report, artifact_folder=artifact_folder)
     return study.best_params
 
 
